#!/usr/bin/env python
from exp_setting import *
import re
from subprocess import Popen, PIPE
import time
from threading import Thread
import sys

CACHE = {'MQ2008' : 1024 * 30, 'MQ2007' : 1024 * 20, 'MQ2007-list' : 1024 * 3}

def read_best_parameter(dataset, targets):
	best_parameter = {}
	for target in targets:
		best_parameter[target] = {}
		with open('/'.join([PARAMETER_DIR, dataset, target, dataset + '.best']), 'r') as f:
			line = f.read()

			result = re.findall('PA (\d+\.\d+) (\d+\.\d+)', line)
			assert len(result) == 1 and len(result[0]) == 2
			result = result[0]
			best_parameter[target][PA] = {C:result[0], G:result[1]}

			result = re.findall('NDCG (\d+\.\d+) (\d+\.\d+)', line)
			assert len(result) == 1 and len(result[0]) == 2
			result = result[0]
			best_parameter[target][NDCG] = {C:result[0], G:result[1]}

			result = re.findall('MAP (\d+\.\d+) (\d+\.\d+)', line)
			assert len(result) == 1 and len(result[0]) == 2
			result = result[0]
			best_parameter[target][MAP] = {C:result[0], G:result[1]}

	return best_parameter

def run_one_target(target, dataset, log_file_path, c, g, time_limit, metric, log_time, iteration_limit):
	cmdline = ''
	train_data_file_path = DATA_PATH[dataset] + '/train.txt'
	test_data_file_path = DATA_PATH[dataset] + '/test.txt'
	model_file_path = EXP_LOG_DIR + '/' + '.'.join([dataset, target, metric, "timelog" if log_time else "log", "model"])

	if target == LIBSVM:
		cmdline = '%s -s 5 -c %s -g %s -t 2 -q -m 30240 -l %s -T %s %s %s' \
			% (PACKAGE_PATH[LIBSVM][TRAIN_LOG_TIME if log_time else TRAIN_LOG],
					c, g, log_file_path, test_data_file_path, train_data_file_path, model_file_path)
	elif target == SVM_LIGHT:
		cmdline = '%s -z p -t 2 -c %s -g %s -m %d -M 0 -N 1 -L %s -E %s -T %d -I %d %s %s' \
			% (PACKAGE_PATH[SVM_LIGHT][TRAIN_LOG_TIME if log_time else TRAIN_LOG],
					c, g, CACHE[dataset], log_file_path, test_data_file_path, time_limit, iteration_limit, train_data_file_path, model_file_path)
	elif target == SVM_RANK:
		cmdline = '%s -v -1 -y -1 -c %lf -g %s -w 4 -f 2000 -t 2 -T %d -L %s -E %s -I %d %s %s' \
				% (PACKAGE_PATH[SVM_RANK][TRAIN_LOG_TIME if log_time else TRAIN_LOG],
					float(c) * NUMBER_OF_QUERY[dataset], g, time_limit, log_file_path, test_data_file_path, iteration_limit, train_data_file_path, model_file_path)

	elif target == RV_SVM:
		cmdline = '%s -r l -t 2 -c %s -g %s -T %d -W 1 -L %s -E %s -I %d %s %s' \
				% (PACKAGE_PATH[RV_SVM][TRAIN_LOG_TIME if log_time else TRAIN_LOG],
					c, g, time_limit, log_file_path, test_data_file_path, iteration_limit, train_data_file_path, model_file_path)

	else:
		assert False

	begin_time = time.time()
	train = Popen(cmdline.split(' '), shell=False, stdout = PIPE, stderr = PIPE)
	stdout, stderr = train.communicate()
	end_time = time.time()
	if len(stderr) > 0:
		print "standard error is not empty!"
		print stderr
	
	return (end_time - begin_time, get_iteration_count(log_file_path))

class Worker (Thread):
	def __init__(self, target, dataset, log_file_path, c, g, time_limit, metric, iteration_limit):
		Thread.__init__(self)
		self.target = target
		self.dataset = dataset
		self.log_file_path = log_file_path
		self.c = c
		self.g = g
		self.time_limit = time_limit
		self.metric = metric
		self.iteration_limit = iteration_limit

	def run(self):
		run_one_target(self.target, self.dataset, self.log_file_path, self.c, self.g, self.time_limit, self.metric, False, self.iteration_limit)

def get_iteration_count(log_file):
	with open(log_file, 'r') as f:
		lines = [ line for line in f.read().split('\n') if len(line) != 0]
		if len(lines) == 0:
			return 0
		last_line = lines[-1]
		iteration = re.findall('iter ([0-9]+):', last_line)[0]
	
	return int(iteration)

def read_itertation_information(file_path):
	iterations = {}
	for metric in METRIC:
		iterations[metric] = {}

	with open(file_path, 'r') as f:
		for line in f.read().split('\n'):
			if len(line) == 0:
				continue
			(target, metric, iteration, c, g) = line.split(' ')
			iterations[metric][target] = int(iteration)
	
	return iterations


def all():
	for dataset, targets in TASK_LIST:
		best_parameter = read_best_parameter(dataset, targets)
		assert targets[0] == LIBSVM


#
		workers = []

		for metric in TARGET_METRIC[dataset]:
			time_limit = 0
			log_files = {}
			iteration_information_file = EXP_LOG_DIR + '/' + '.'.join([dataset, metric, 'iteration'])
			iterations = read_itertation_information(iteration_information_file)
			
			for target in targets:
				log_file_path = EXP_LOG_DIR + '/' + '.'.join([dataset, target, metric, 'log'])
				worker = Worker(target, dataset, log_file_path, best_parameter[target][metric][C], best_parameter[target][metric][G], -1, metric, iterations[metric][target])
				worker.start()
				worker.join()
#				workers.append(worker)

#		for worker in workers:
#			worker.join()

def specify(argv):

	(dataset, metric) = argv[1:3]
	targets = TARGETS[dataset]
	best_parameter = read_best_parameter(dataset, targets)

	iteration_information_file = EXP_LOG_DIR + '/' + '.'.join([dataset, metric, 'iteration'])

	iterations = read_itertation_information(iteration_information_file)

	workers = []

	
	for target in targets:
		log_file_path = EXP_LOG_DIR + '/' + '.'.join([dataset, target, metric, 'log'])
		worker = Worker(target, dataset, log_file_path, best_parameter[target][metric][C], best_parameter[target][metric][G], -1, metric, iterations[metric][target])
		worker.start()
		workers.append(worker)

	for worker in workers:
		worker.join()


def specify_package(argv):

	(dataset, metric, target) = argv[1:4]
	targets = [target]
	best_parameter = read_best_parameter(dataset, targets)

	iteration_information_file = EXP_LOG_DIR + '/' + '.'.join([dataset, metric, 'iteration'])

	iterations = read_itertation_information(iteration_information_file)

	workers = []

	
	for target in targets:
		log_file_path = EXP_LOG_DIR + '/' + '.'.join([dataset, target, metric, 'log'])
		worker = Worker(target, dataset, log_file_path, best_parameter[target][metric][C], best_parameter[target][metric][G], -1, metric, iterations[metric][target])
		worker.start()
		workers.append(worker)

	for worker in workers:
		worker.join()







if __name__ == '__main__':
	if len(sys.argv) == 1 + 2:
		sys.exit(specify(sys.argv))
	elif len(sys.argv) == 1 + 3:
		sys.exit(specify_package(sys.argv))
	elif len(sys.argv) == 1:
		sys.exit(all())
	else:
		print >> sys.stderr, 'Unknown target\n'
		sys.exit(-1)
